@@ -8,10 +8,9 @@ use Carp ();
 use Devel::Symdump;
 use Digest::MD5;
 use Graph::Easy;
-use List::MoreUtils ();
+use List::MoreUtils  ();
 use Sub::Information ();
 use Text::SimpleTable;
-use Tree;
 
 =head1 NAME
 
@@ -19,11 +18,11 @@ Class::Sniff - Look for class composition code smells
 
 =head1 VERSION
 
-Version 0.06
+Version 0.07
 
 =cut
 
-our $VERSION = '0.06';
+our $VERSION = '0.07';
 
 =head1 SYNOPSIS
 
@@ -87,6 +86,9 @@ The name of the class to sniff.  If the class is not loaded into memory, the
 constructor will still work, but nothing will get reported.  You must ensure
 that your class is already loaded!
 
+If you pass it an instance of a class instead, it will call 'ref' on the class
+to determine what class to use.
+
 =item * C<ignore>
 
 Optional.
@@ -116,13 +118,13 @@ for details.
 
 sub new {
     my ( $class, $arg_for ) = @_;
-    my $target_class = $arg_for->{class}
+    my $proto = $arg_for->{class}
       or Carp::croak("'class' argument not supplied to 'new'");
+    my $target_class = ref $proto || $proto;
     if ( exists $arg_for->{ignore} && 'Regexp' ne ref $arg_for->{ignore} ) {
         Carp::croak("'ignore' requires a regex");
     }
     my $self = bless {
-        class_order   => {},
         classes       => {},
         duplicates    => {},
         exported      => {},
@@ -133,7 +135,6 @@ sub new {
         methods       => {},
         paths         => [ [$target_class] ],
         target        => $target_class,
-        tree          => undef,
         universal     => $arg_for->{universal},
         method_length => ( $arg_for->{method_length} || 50 ),
     } => $class;
@@ -147,17 +148,10 @@ sub _initialize {
     $self->width(72);
     $self->_register_class($target_class);
     $self->{classes}{$target_class}{count} = 1;
-    $self->{tree} = Tree->new($target_class);
-    $self->_build_tree( $self->tree );
+    $self->{graph} = Graph::Easy->new;
+    $self->{graph}->set_attribute( 'graph', 'flow', 'up' );
+    $self->_build_hierarchy($target_class);
 
-    my $graph = Graph::Easy->new;
-    for my $node ( $self->tree->traverse ) {
-        my $class = $node->value;
-        next if $class eq $target_class;
-        $graph->add_edge_once( $node->parent->value, $class );
-    }
-    $graph->set_attribute( 'graph', 'flow', 'up' );
-    $self->{graph} = $graph;
     $self->_finalize;
 }
 
@@ -171,7 +165,6 @@ sub _finalize {
     while ( my ( $method, $classes ) = each %{ $self->{methods} } ) {
         @$classes = sort { $classes{$a} <=> $classes{$b} } @$classes;
     }
-    $self->{class_order} = \%classes;
 }
 
 sub _register_class {
@@ -184,15 +177,19 @@ sub _register_class {
 
     foreach my $method (@methods) {
         my $coderef = $class->can($method)
-            or Carp::croak("Panic: $class->can($method) returned false!");
+          or Carp::croak("Panic: $class->can($method) returned false!");
         my $info = Sub::Information::inspect($coderef);
         if ( $info->package ne $class ) {
             $self->{exported}{$class}{$method} = $info->package;
         }
         else {
+
+            # It's OK to throw away the exception.  The B:: modules can be
+            # tricky and this is documented as experimental.
+            local $@;
             eval {
-                my $line = $info->line;
-                my $length  = B::svref_2object($coderef)->GV->LINE - $line;
+                my $line   = $info->line;
+                my $length = B::svref_2object($coderef)->GV->LINE - $line;
                 if ( $length > $self->method_length ) {
                     $self->{long_methods}{"$class\::$method"} = $length;
                 }
@@ -200,13 +197,13 @@ sub _register_class {
         }
 
         my $walker = B::Concise::compile( '-terse', $coderef );    # 1
-        B::Concise::walk_output( \my $buffer);
+        B::Concise::walk_output( \my $buffer );
         $walker->();    # 1 renders -terse
-        $buffer =~ s/^.*//;   # strip method name
-        $buffer =~ s/\(0x[^)]+\)/(0xHEXNUMBER)/g;   # normalize addresses
+        $buffer =~ s/^.*//;                          # strip method name
+        $buffer =~ s/\(0x[^)]+\)/(0xHEXNUMBER)/g;    # normalize addresses
         my $digest = Digest::MD5::md5_hex($buffer);
         $self->{duplicates}{$digest} ||= [];
-        push @{ $self->{duplicates}{$digest} } => [$class, $method];
+        push @{ $self->{duplicates}{$digest} } => [ $class, $method ];
     }
 
     for my $method (@methods) {
@@ -464,8 +461,8 @@ removed.  You may feel OK with this if the duplicated methods are exported
 sub duplicate_methods {
     my $self = shift;
     my @duplicates;
-    foreach my $methods ( values%{ $self->{duplicates} } ) {
-        if (@$methods > 1) {
+    foreach my $methods ( values %{ $self->{duplicates} } ) {
+        if ( @$methods > 1 ) {
             push @duplicates => $methods;
         }
     }
@@ -619,17 +616,18 @@ sub report {
 sub _get_duplicate_method_report {
     my $self = shift;
 
-    my $report = '';
+    my $report    = '';
     my @duplicate = $self->duplicate_methods;
-    my (@methods, @duplicates);
-    if ( @duplicate ) {
+    my ( @methods, @duplicates );
+    if (@duplicate) {
         foreach my $duplicate (@duplicate) {
             push @methods => join '::' => @{ pop @$duplicate };
-            push @duplicates => join "\n" => map { join '::' => @$_ } @$duplicate;
+            push @duplicates => join "\n" => map { join '::' => @$_ }
+              @$duplicate;
         }
         $report .= "Duplicate Methods (Experimental)\n"
-          . $self->_build_report( 'Method', 'Duplicated In', \@methods,
-          \@duplicates );
+          . $self->_build_report( 'Method', 'Duplicated In',
+            \@methods, \@duplicates );
     }
     return $report;
 }
@@ -637,9 +635,9 @@ sub _get_duplicate_method_report {
 sub _get_overridden_report {
     my $self = shift;
 
-    my $report = '';
+    my $report     = '';
     my $overridden = $self->overridden;
-    if ( %$overridden ) {
+    if (%$overridden) {
         my @methods = sort keys %$overridden;
         my @classes;
         foreach my $method (@methods) {
@@ -680,16 +678,16 @@ sub _get_multiple_inheritance_report {
 }
 
 sub _get_exported_report {
-    my $self = shift;
+    my $self     = shift;
     my $exported = $self->exported;
-    my $report = '';
+    my $report   = '';
     if ( my @classes = sort keys %$exported ) {
-        my ($longest_c, $longest_m) = (length('Class'), length('Method') );
-        my (@subs,@sources);
+        my ( $longest_c, $longest_m ) = ( length('Class'), length('Method') );
+        my ( @subs, @sources );
         foreach my $class (@classes) {
-            my (@temp_subs, @temp_sources);
-            foreach my $sub (sort keys %{ $exported->{$class} } ) {
-                push @temp_subs => $sub;
+            my ( @temp_subs, @temp_sources );
+            foreach my $sub ( sort keys %{ $exported->{$class} } ) {
+                push @temp_subs    => $sub;
                 push @temp_sources => $exported->{$class}{$sub};
                 $longest_c = length($class) if length($class) > $longest_c;
                 $longest_m = length($sub)   if length($sub) > $longest_m;
@@ -698,10 +696,10 @@ sub _get_exported_report {
             push @sources => join "\n" => @temp_sources;
         }
         my $width = $self->width - 3;
-        my $third = int($width/3);
+        my $third = int( $width / 3 );
         $longest_c = $third if $longest_c > $third;
         $longest_m = $third if $longest_m > $third;
-        my $rest = $width - ($longest_c + $longest_m);
+        my $rest = $width - ( $longest_c + $longest_m );
         my $text = Text::SimpleTable->new(
             [ $longest_c, 'Class' ],
             [ $longest_m, 'Method' ],
@@ -710,7 +708,7 @@ sub _get_exported_report {
         for my $i ( 0 .. $#classes ) {
             $text->row( $classes[$i], $subs[$i], $sources[$i] );
         }
-        $report .= "Exported Subroutines\n".$text->draw;
+        $report .= "Exported Subroutines\n" . $text->draw;
     }
     return $report;
 }
@@ -748,14 +746,14 @@ sub _build_report {
 sub _get_widths {
     my ( $self, $title, @strings ) = @_;
 
-    my $width = $self->width;
+    my $width   = $self->width;
     my $longest = length($title);
     foreach my $string (@strings) {
         my $length = length $string;
         $longest = $length if $length > $longest;
     }
-    $longest = int( $width / 2 ) if $longest > ($width / 2);
-    return ($longest, $width - $longest);
+    $longest = int( $width / 2 ) if $longest > ( $width / 2 );
+    return ( $longest, $width - $longest );
 }
 
 =head2 C<width>
@@ -790,16 +788,6 @@ force it to respect the order in which classes are ordered.  Thus, the
 
 sub to_string { $_[0]->graph->as_ascii }
 
-=head2 C<tree>
-
- my $tree = $sniff->tree;
-
-Returns a L<Tree> representation of the inheritance hierarchy.
-
-=cut
-
-sub tree { $_[0]->{tree} }
-
 =head2 C<graph>
 
  my $graph = $sniff->graph;
@@ -823,6 +811,38 @@ right.
 
 sub graph { $_[0]->{graph} }
 
+=head2 C<combine_graphs>
+
+ my $graph = $sniff->combine_graphs($sniff2, $sniff3);
+ print $graph->as_ascii;
+
+Allows you to create a large inheritance hierarchy graph by combining several
+C<Class::Sniff> instances together.
+
+Returns a L<Graph::Easy> object.
+
+=cut
+
+sub combine_graphs {
+    my ( $self, @sniffs ) = @_;
+
+    my $graph = $self->graph->copy;
+
+    foreach my $sniff (@sniffs) {
+        unless ( $sniff->isa( ref $self ) ) {
+            my $bad_class = ref $sniff;
+            my $class     = ref $self;
+            die
+"Arguments to 'combine_graphs' must '$class' objects, not '$bad_class' objects";
+        }
+        my $next_graph = $sniff->graph;
+        foreach my $edge ( $next_graph->edges ) {
+            $graph->add_edge_once( $edge->from->name, $edge->to->name );
+        }
+    }
+    return $graph;
+}
+
 =head2 C<target_class>
 
  my $class = $sniff->target_class;
@@ -929,8 +949,7 @@ sub methods {
 }
 
 sub _get_parents {
-    my ( $self, $node ) = @_;
-    my $class = $node->value;
+    my ( $self, $class ) = @_;
     return if $class eq 'UNIVERSAL';
     no strict 'refs';
 
@@ -945,17 +964,15 @@ sub _get_parents {
 }
 
 # This is the heart of where we set just about everything up.
-sub _build_tree {
-    my ( $self, @nodes ) = @_;
+sub _build_hierarchy {
+    my ( $self, @classes ) = @_;
 
-    for my $node (@nodes) {
-        return unless $self->_get_parents($node);
-        foreach my $class ( $node->value, $self->_get_parents($node) ) {
-            $self->_register_class($class);
-        }
-        $self->_add_children($node);
-        $self->_build_paths($node);
-        $self->_add_parents($node);
+    for my $class (@classes) {
+        return unless my @parents = $self->_get_parents($class);
+        $self->_register_class($_) foreach $class, @parents;
+        $self->_add_children($class);
+        $self->_build_paths($class);
+        $self->_add_parents($class);
     }
 }
 
@@ -963,11 +980,10 @@ sub _build_tree {
 # will take through the code to find a method.  This is based on Perl's
 # default search order, not C3.
 sub _build_paths {
-    my ( $self, $node) = @_;
+    my ( $self, $class ) = @_;
+
+    my @parents = $self->_get_parents($class);
 
-    my $class = $node->value;
-    my @parents = $self->_get_parents($node);
-    
     # XXX strictly speaking, we can skip $do_chg, but if path() get's
     # expensive (such as testing for valid classes), then we
     # need it.
@@ -994,26 +1010,27 @@ sub _build_paths {
 }
 
 sub _add_parents {
-    my ( $self, $node ) = @_;
+    my ( $self, $class ) = @_;
 
     # This algorithm will follow classes in Perl's default inheritance
     # order
-    foreach my $parent ($self->_get_parents($node)) {
+    foreach my $parent ( $self->_get_parents($class) ) {
         push @{ $self->{list_classes} } => $parent
           unless grep { $_ eq $parent } @{ $self->{list_classes} };
         $self->{classes}{$parent}{count}++;
-        my $tree = Tree->new($parent);
-        $node->add_child($tree);
-        $self->_build_tree($tree);
+        $self->_build_hierarchy($parent);
     }
 }
+
 sub _add_children {
-    my ( $self, $node ) = @_;
-    my $class   = $node->value;
-    my @parents = $self->_get_parents($node);
+    my ( $self, $class ) = @_;
+    my @parents = $self->_get_parents($class);
 
     $self->{classes}{$class}{parents} = \@parents;
-    $self->_add_child( $_, $class ) foreach @parents;
+    foreach my $parent (@parents) {
+        $self->_add_child( $parent, $class );
+        $self->graph->add_edge_once( $class, $parent );
+    }
     return $self;
 }
 
